@@ -1,4 +1,5 @@
 # -*- coding: utf-8 -*-
+from pathlib import Path
 from urllib.parse import quote_plus
 
 from plexapi import media, utils
@@ -8,8 +9,7 @@ from plexapi.library import LibrarySection, ManagedHub
 from plexapi.mixins import (
     AdvancedSettingsMixin, SmartFilterMixin, HubsMixin, RatingMixin,
     ArtMixin, PosterMixin, ThemeMixin,
-    AddedAtMixin, ContentRatingMixin, SortTitleMixin, SummaryMixin, TitleMixin,
-    LabelMixin
+    CollectionEditMixins
 )
 from plexapi.utils import deprecated
 
@@ -19,8 +19,7 @@ class Collection(
     PlexPartialObject,
     AdvancedSettingsMixin, SmartFilterMixin, HubsMixin, RatingMixin,
     ArtMixin, PosterMixin, ThemeMixin,
-    AddedAtMixin, ContentRatingMixin, SortTitleMixin, SummaryMixin, TitleMixin,
-    LabelMixin
+    CollectionEditMixins
 ):
     """ Represents a single Collection.
 
@@ -222,6 +221,7 @@ class Collection(
                 .. code-block:: python
 
                     collection.updateMode(user="user")
+
         """
         if not self.smart:
             raise BadRequest('Cannot change collection filtering user for a non-smart collection.')
@@ -250,6 +250,7 @@ class Collection(
                 .. code-block:: python
 
                     collection.updateMode(mode="hide")
+
         """
         mode_dict = {
             'default': -1,
@@ -275,7 +276,8 @@ class Collection(
 
                 .. code-block:: python
 
-                    collection.updateSort(mode="alpha")
+                    collection.sortUpdate(sort="alpha")
+
         """
         if self.smart:
             raise BadRequest('Cannot change collection order for a smart collection.')
@@ -398,7 +400,7 @@ class Collection(
     @deprecated('use editTitle, editSortTitle, editContentRating, and editSummary instead')
     def edit(self, title=None, titleSort=None, contentRating=None, summary=None, **kwargs):
         """ Edit the collection.
-        
+
             Parameters:
                 title (str, optional): The title of the collection.
                 titleSort (str, optional): The sort title of the collection.
@@ -501,6 +503,8 @@ class Collection(
                 :class:`~plexapi.collection.Collection`: A new instance of the created Collection.
         """
         if smart:
+            if items:
+                raise BadRequest('Cannot create a smart collection with items.')
             return cls._createSmart(server, title, section, limit, libtype, sort, filters, **kwargs)
         else:
             return cls._create(server, title, section, items)
@@ -559,3 +563,9 @@ class Collection(
             raise Unsupported('Unsupported collection content')
 
         return myplex.sync(sync_item, client=client, clientId=clientId)
+
+    @property
+    def metadataDirectory(self):
+        """ Returns the Plex Media Server data directory where the metadata is stored. """
+        guid_hash = utils.sha1hash(self.guid)
+        return str(Path('Metadata') / 'Collections' / guid_hash[0] / f'{guid_hash[1:]}.bundle')
