@@ -1,12 +1,13 @@
 # -*- coding: utf-8 -*-
 import re
+from pathlib import Path
 from urllib.parse import quote_plus, unquote
 
 from plexapi import media, utils
 from plexapi.base import Playable, PlexPartialObject
 from plexapi.exceptions import BadRequest, NotFound, Unsupported
 from plexapi.library import LibrarySection, MusicSection
-from plexapi.mixins import SmartFilterMixin, ArtMixin, PosterMixin
+from plexapi.mixins import SmartFilterMixin, ArtMixin, PosterMixin, PlaylistEditMixins
 from plexapi.utils import deprecated
 
 
@@ -14,7 +15,8 @@ from plexapi.utils import deprecated
 class Playlist(
     PlexPartialObject, Playable,
     SmartFilterMixin,
-    ArtMixin, PosterMixin
+    ArtMixin, PosterMixin,
+    PlaylistEditMixins
 ):
     """ Represents a single Playlist.
 
@@ -41,6 +43,7 @@ class Playlist(
             smart (bool): True if the playlist is a smart playlist.
             summary (str): Summary of the playlist.
             title (str): Name of the playlist.
+            titleSort (str): Title to use when sorting (defaults to title).
             type (str): 'playlist'
             updatedAt (datetime): Datetime the playlist was updated.
     """
@@ -70,6 +73,7 @@ class Playlist(
         self.smart = utils.cast(bool, data.attrib.get('smart'))
         self.summary = data.attrib.get('summary')
         self.title = data.attrib.get('title')
+        self.titleSort = data.attrib.get('titleSort', self.title)
         self.type = data.attrib.get('type')
         self.updatedAt = utils.toDatetime(data.attrib.get('updatedAt'))
         self._items = None  # cache for self.items
@@ -154,7 +158,7 @@ class Playlist(
                 sectionKey = int(match.group(1))
                 self._section = self._server.library.sectionByID(sectionKey)
                 return self._section
-        
+
             # Try to get the library section from the first item in the playlist
             if self.items():
                 self._section = self.items()[0].section()
@@ -223,7 +227,7 @@ class Playlist(
         self._server.query(key, method=self._server._session.put)
         return self
 
-    @deprecated('use "removeItems" instead', stacklevel=3)
+    @deprecated('use "removeItems" instead')
     def removeItem(self, item):
         self.removeItems(item)
 
@@ -307,13 +311,18 @@ class Playlist(
 
     def _edit(self, **kwargs):
         """ Actually edit the playlist. """
+        if isinstance(self._edits, dict):
+            self._edits.update(kwargs)
+            return self
+
         key = f'{self.key}{utils.joinArgs(kwargs)}'
         self._server.query(key, method=self._server._session.put)
         return self
 
+    @deprecated('use "editTitle" and "editSummary" instead')
     def edit(self, title=None, summary=None):
         """ Edit the playlist.
-        
+
             Parameters:
                 title (str, optional): The title of the playlist.
                 summary (str, optional): The summary of the playlist.
@@ -383,7 +392,7 @@ class Playlist(
         key = f"/playlists/upload{utils.joinArgs(args)}"
         server.query(key, method=server._session.post)
         try:
-            return server.playlists(sectionId=section.key, guid__endswith=m3ufilepath)[0].edit(title=title).reload()
+            return server.playlists(sectionId=section.key, guid__endswith=m3ufilepath)[0].editTitle(title).reload()
         except IndexError:
             raise BadRequest('Failed to create playlist from m3u file.') from None
 
@@ -425,15 +434,18 @@ class Playlist(
         if m3ufilepath:
             return cls._createFromM3U(server, title, section, m3ufilepath)
         elif smart:
+            if items:
+                raise BadRequest('Cannot create a smart playlist with items.')
             return cls._createSmart(server, title, section, limit, libtype, sort, filters, **kwargs)
         else:
             return cls._create(server, title, items)
 
     def copyToUser(self, user):
         """ Copy playlist to another user account.
-        
+
             Parameters:
-                user (str): Username, email or user id of the user to copy the playlist to.
+                user (:class:`~plexapi.myplex.MyPlexUser` or str): `MyPlexUser` object, username,
+                    email, or user id of the user to copy the playlist to.
         """
         userServer = self._server.switchUser(user)
         return self.create(server=userServer, title=self.title, items=self.items())
@@ -495,3 +507,9 @@ class Playlist(
     def _getWebURL(self, base=None):
         """ Get the Plex Web URL with the correct parameters. """
         return self._server._buildWebURL(base=base, endpoint='playlist', key=self.key)
+
+    @property
+    def metadataDirectory(self):
+        """ Returns the Plex Media Server data directory where the metadata is stored. """
+        guid_hash = utils.sha1hash(self.guid)
+        return str(Path('Metadata') / 'Playlists' / guid_hash[0] / f'{guid_hash[1:]}.bundle')
